"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("@ge-ge/utils");
var Highlight = /** @class */ (function () {
    /**
     *
     * @param {{className?: string; theme?: string}} options
     */
    function Highlight(options) {
        this.THEME = 'highlight-theme';
        if (options) {
            Highlight.CLASS_NAME = options.className || Highlight.CLASS_NAME;
            this.THEME = options.theme || this.THEME;
        }
    }
    /**
     *
     * @param {HTMLElement} ele
     * @description 将<mark>this is text content</mark> 转为 文字，并插入到原来的位置
     */
    Highlight.reset = function (ele) {
        if (ele.nodeName === 'MARK' && ele.parentNode) {
            ele.parentNode.textContent = ele.parentNode.textContent;
        }
    };
    /**
     *
     * @description 使传入的Node节点高亮
     * @param {Node} ele   高亮的元素
     * @param {Node} start 开始高亮的元素
     * @param {Node} end   结束高亮的元素
     */
    Highlight.prototype.highLight = function (ele, start, end) {
        var node = ele;
        var flag = start && end ? false : true; // 是否一开始就进行高亮处理
        var markNode = []; // 高亮后的元素
        var need_mark = []; // 待高亮处理的元素，包含mark元素
        var recursion = function (node) {
            if (end && node === end) {
                flag = false;
            }
            if (flag && node.nodeName === 'MARK') {
                // 对只含有Text的mark元素不处理
                if (!(node.children.length)) {
                    need_mark.push(node);
                    return;
                }
            }
            if (node.hasChildNodes()) {
                // NodeList is not an Array,
                var length_1 = node.childNodes.length;
                for (var i = 0; i < length_1; i++) {
                    recursion(node.childNodes[i]);
                }
            }
            else {
                // 不对mark元素内的文字再次进行高亮
                if (flag && node.parentNode && node.parentNode.nodeName === 'MARK') {
                    return;
                }
                if (flag && node.nodeType === Node.TEXT_NODE && node.textContent) {
                    if (node.textContent.trim().length === 0)
                        return;
                    need_mark.push(node);
                }
            }
            if (start && node === start) {
                flag = true;
            }
        };
        recursion(node);
        for (var _i = 0, need_mark_1 = need_mark; _i < need_mark_1.length; _i++) {
            var node_1 = need_mark_1[_i];
            if (node_1.nodeType === Node.TEXT_NODE && node_1.textContent) {
                // 不对mark元素内的文字再次进行高亮
                if (node_1.parentNode && node_1.parentNode.nodeName === 'MARK')
                    continue;
                var mark = this.highLightText(node_1);
                var obj = { mix: false, el: mark };
                markNode.push(obj);
            }
            else if (node_1.nodeName === 'MARK') {
                var obj = { mix: true, el: node_1 };
                markNode.push(obj);
            }
        }
        need_mark = [];
        return markNode;
    };
    /**
     * @description 是否为文本节点
     * @param {Node} node
     * @returns {boolean}
     */
    Highlight.isText = function (node) {
        if (node.nodeType === Node.TEXT_NODE) {
            return true;
        }
        return false;
    };
    /**
     * @description 使TextNode高亮
     * @param {Text} node     文本节点
     * @param {number} start  开始的位置，默认从0开始
     * @param {number} count  高亮文字的数量，默认为 从开始位置之后的全部文字
     */
    Highlight.prototype.highLightText = function (node, start, count) {
        if (start === void 0) { start = 0; }
        if (node.nodeType === Node.TEXT_NODE && node.textContent) {
            if (node.textContent.trim().length === 0)
                return;
            count = count || node.textContent.trim().length - start;
            var textContent = node.textContent.substr(start, count);
            var mark = this.getMarkElement(textContent); // 生成highlight的元素
            var range = document.createRange();
            range.setStart(node, start);
            range.setEnd(node, start + count); // TODO：把选取内的文字移入mark，而不是删除
            range.deleteContents(); // 删除选中的文字，之后插入高亮元素
            if (mark)
                range.insertNode(mark);
            range.detach();
            // console.log(mark)
            return mark;
        }
    };
    /**
     *
     * @description 返回高亮的mark Element
     * @param {string} text
     * @returns {HTMLElement}
     */
    Highlight.prototype.getMarkElement = function (text) {
        var content = text.trim();
        var mark = document.createElement('mark');
        var class_list = new utils_1.ClassList(mark);
        class_list.addClass(Highlight.CLASS_NAME); // 添加默认className
        class_list.addClass(this.THEME); // 添加自定义className
        mark.textContent = content;
        return mark;
    };
    Highlight.CLASS_NAME = 'highlight';
    return Highlight;
}());
exports.default = Highlight;
