"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var markRange_1 = require("./markRange");
var highlight_1 = require("./highlight");
var utils_1 = require("@ge-ge/utils");
var Mark = /** @class */ (function () {
    /**
     * @param {Range} range
     */
    function Mark(options) {
        this.range = document.createRange();
        this.highlight = new highlight_1.default(options);
        this.marks = [];
        this.start = null;
        this.end = null;
    }
    /**
     * @description 高亮选中的区间
     */
    Mark.prototype.mark = function (range) {
        var _this = this;
        var new_range_list = markRange_1.default.splitRange(range); // 分割成3个range
        new_range_list.forEach(function (new_range) {
            var docFragment = new_range.range.extractContents(); // 将选区内的元素移出到documentFragment
            // 对docFragment进行处理
            if (new_range.id === 'center' && _this.start && _this.end) {
                var markNode = _this.highlight.highLight(docFragment, _this.start, _this.end);
                // this.marks = this.marks.concat(markNode)
                (_a = _this.marks).splice.apply(_a, [1, 0].concat(markNode));
            }
            else {
                var markNode = _this.highlight.highLight(docFragment);
                // this.marks = this.marks.concat(markNode)
                if (new_range.id === 'start') {
                    _this.start = markNode[0].el;
                    (_b = _this.marks).unshift.apply(_b, markNode);
                }
                else if (new_range.id === 'end') {
                    _this.end = markNode[0].el;
                    (_c = _this.marks).push.apply(_c, markNode);
                }
            }
            // 处理完成后插入到对应的range,range被修改没有参考意义
            new_range.range.insertNode(docFragment);
            var _a, _b, _c;
        });
        // 设置高亮后的range
        if (this.start && this.end) {
            this.range.setStartBefore(this.start);
            this.range.setEndAfter(this.end);
        }
        Mark.ID++;
    };
    /**
     * @description 恢复到没有高亮之前的样子
     */
    Mark.prototype.reset = function () {
        // TODO 改为根据marks恢复
        for (var _i = 0, _a = this.marks; _i < _a.length; _i++) {
            var mark = _a[_i];
            highlight_1.default.reset(mark.el);
        }
        this.start = this.end = null;
        this.marks = [];
        this.range.detach();
    };
    /**
     * @description 为一个节点添加class名,并且返回被添加的node
     * @param {Node} ele
     * @param {string} id
     * @returns {Node}
     */
    Mark.markID = function (ele, id) {
        var r = function (node) {
            if (highlight_1.default.isText(node) && node.parentNode) {
                r(node.parentNode);
            }
            else {
                var class_list = new utils_1.ClassList(node);
                class_list.addClass(id);
            }
            return node;
        };
        return r(ele);
    };
    Mark.PREFIX = 'mark--highlight__';
    Mark.START = Mark.PREFIX + 'start';
    Mark.END = Mark.PREFIX + 'end';
    Mark.ID = 0;
    return Mark;
}());
exports.default = Mark;
